package main

import "fmt"

type exit struct {
	dir  string
	from *room
	to   *room
}

type room struct {
	id    uint64
	short string // short description of room
	long  string // long description
	exits map[string]exit
}

func connect(from *room, to *room, dir string) {
	from.exits[dir] = exit{from: from, to: to, dir: dir}
}

// This also tracks a room's coordinates and its parent (from).
type parent struct {
	from *room
	x    int
	y    int
}

func coord(dir string, x int, y int) (int, int) {
	switch dir {
	case "north":
		y += 1
	case "south":
		y -= 1
	case "west":
		x -= 1
	case "east":
		x += 1
	}
	return x, y
}

func bfs(r *room) []string {
	queue := []*room{r}
	visited := make(map[uint64]bool)
	traveled := make(map[*room]parent)
	traveled[r] = parent{r, 0, 0}
	for len(queue) > 0 {
		lsize := len(queue)

		for lsize != 0 {
			lsize--
			current := queue[0]
			queue = queue[1:]
			visited[current.id] = true

			for dir, e := range current.exits {
				if _, seen := visited[e.to.id]; !seen {
					x, y := coord(dir, traveled[current].x, traveled[current].y)
					traveled[e.to] = parent{current, x, y}
					queue = append(queue, e.to)
				}
			}
		}
	}
	result := make([]string, 0)
	for k, v := range traveled {
		// fmt.Printf("dir: %s --> [%d,%d]\n", k.short, v.x, v.y)
		result = append(result, fmt.Sprintf("room: %s, coords: [%d,%d]\n", k.short, v.x, v.y))
	}
	return result

}

func main() {
	rooms := []room{
		room{0, "center", "center long", make(map[string]exit)},
		room{1, "street e", "a street long", make(map[string]exit)},
		room{2, "street e", "a street long", make(map[string]exit)},
		room{3, "street e", "a street long", make(map[string]exit)},
		room{4, "armor shop", "armor shop long", make(map[string]exit)},
		room{5, "street w", "a street long", make(map[string]exit)},
	}

	// Connect some rooms like below:
	//                 [4]
	//                  |
	// [5]-[0]-[1]-[2]-[3]
	connect(&rooms[0], &rooms[1], "east")
	connect(&rooms[1], &rooms[0], "west")

	connect(&rooms[0], &rooms[5], "west")
	connect(&rooms[5], &rooms[0], "east")

	connect(&rooms[1], &rooms[2], "east")
	connect(&rooms[2], &rooms[1], "west")

	connect(&rooms[2], &rooms[3], "east")
	connect(&rooms[3], &rooms[2], "west")

	connect(&rooms[3], &rooms[4], "north")
	connect(&rooms[4], &rooms[3], "south")

	roomCoords := bfs(&rooms[0])
	for _, c := range roomCoords {
		fmt.Print(c)
	}
}
